# -*- coding: cp1252 -*-
from xml.dom import minidom, Node
import codecs
import math

#***********************************************************************************************************************************************************#
#********                                       OSM to tree Class                                                                                   ********#
#***********************************************************************************************************************************************************#
# This class converts a .OSM file into a tree structure and gives methods to work with it                                                                   #

class osmTree():

    def __init__(self,osmFile):

        if osmFile!='':
            osmMap=open(osmFile,"r")
            self.root=minidom.parseString(self.clean(osmMap))
        else:
            string="<?xml version=" + chr(34) + "1.0" + chr(34)+ " encoding="+chr(34) +"UTF-8"+chr(34)+"?>\n <osm version="+ chr(34) +"0.6"+ chr(34) +" generator="+ chr(34) +"OpenStreetMap server"+ chr(34) +"> \n </osm>"
            self.root=minidom.parseString(string)

    def addStreet(self,NodeList,ID):
        osm=self.root.childNodes[0]
        way=osm.appendChild(self.root.createElement("way"))
        way.setAttribute("id",str(ID))
        way.setAttribute("visible","true")
        for Node in NodeList:
            node=osm.appendChild(self.root.createElement("nd"))
            node.setAttribute("ref",Node)
            way.appendChild(node)
        node=osm.appendChild(self.root.createElement("tag"))
        node.setAttribute("k","name")
        node.setAttribute("v",("Street"+str(ID)))
        way.appendChild(node)
        node=osm.appendChild(self.root.createElement("tag"))
        node.setAttribute("k","highway")
        node.setAttribute("v","residential")
        way.appendChild(node)

    def addNodes(self,NodeDic):
        osm=self.root.childNodes[0]
        NodeList=NodeDic.keys()
        for element in NodeList:
            Node=osm.appendChild(self.root.createElement("node"))
            Node.setAttribute("id",element)
            Node.setAttribute("lat",NodeDic[element][1])
            Node.setAttribute("lon",NodeDic[element][0])

    def setBounds(self):
        minLat,minLon,maxLat,maxLon=self.getMinMax()
        osm=self.root.childNodes[0]
        Node=osm.insertBefore(self.root.createElement("bounds"),osm.firstChild)
        Node.setAttribute("minlat",minLat)
        Node.setAttribute("minlon",minLon)
        Node.setAttribute("maxlat",maxLat)
        Node.setAttribute("maxlon",maxLon)
        

    def getMinMax(self):
        Nodes=self.getNodeList()
        minLat=Nodes[0].getAttribute('lat')
        minLon=Nodes[0].getAttribute('lon')
        maxLat=Nodes[0].getAttribute('lat')
        maxLon=Nodes[0].getAttribute('lon')
        for node in Nodes:
            lon=node.getAttribute('lon')
            lat=node.getAttribute('lat')
            if lon>maxLon:
                maxLon=lon
            elif lon<minLon:
                minLon=lon
            if lat>maxLat:
                maxLat=lat
            elif lat<minLat:
                minLat=lat
        return minLat,minLon,maxLat,maxLon
        
    def clean(self,fileToClean):
        """
             Erase the possible \n, \t, and "    " elements in the text
        """
        openFile=fileToClean.read()
        without_n= openFile.replace('\n','')
        without_t= without_n.replace('\t','')
        without_s= without_t.replace('    ','')
        return without_s

    def getStreetsList(self):
        """
            returs a List with all the Ways (streets and other elements)
        """
        return self.root.getElementsByTagName('way')

    def getNodeList(self):
        """
            returns a List of all Nodes
        """
        return self.root.getElementsByTagName('node')
    
    def deleteBuildings(self):
        """
           Given a Type, it deletes all the ways which have it     
        """
        ways=self.getStreetsList()
        i=0
        while i<len(ways):
            tags=ways[i].getElementsByTagName('tag')
            j=0
            while j<len(tags):
                if(tags[j].getAttribute('k')==u'building'):
                    self.eraseNode(ways[i])
                    j=len(tags)
                else:
                    j +=1
            i +=1


    def eraseNoTypeElements(self,Type):
        """
            Given a Type, it deletes all the ways which don't have it
        """

        ways=self.getStreetsList()
        isHighWay= 0
        i=0
        while i<len(ways):
            tags=ways[i].getElementsByTagName('tag')
            j=0
            while j<len(tags):
                if(tags[j].getAttribute('k')==Type):
                    isHighWay= 1
                    j=len(tags)
                else:
                    j +=1
            if isHighWay== 0:
                self.eraseNode(ways[i])
            else:
                isHighWay=0
            i +=1

    def erase(self,objectList):
        """
            erase from the structure the elements in objectList
        """
        father=''
        for element in objectList:
            father=element.parentNode
            father.removeChild(element)
            
    def eraseNode(self,Node):
        """
            erase the given Node from the structure
        """
        father = Node.parentNode
        father.removeChild(Node)
    
    def getIntersectionNodes(self):
        """
            Returns a List with all the intersection nodes ID
        """
        streets=self.getStreetsList()
        nodeList=[]
        intersectionList=[]
        for nodeElement in streets:
            nodes=nodeElement.getElementsByTagName('nd')
            for node in nodes:
                nodeList.append(node.getAttribute('ref'))
        for element in nodeList:
            if nodeList.count(element)>1 and intersectionList.count(element)==0:
                intersectionList.append(element)
        return intersectionList

    def getNode(self,nodeID):
        """
            Return the Node wich have the given ID
        """
        
        Nlist=self.getNodeList()
        for element in Nlist:
            if element.getAttribute('id')==nodeID:
                return element

    def deleteNodeMarks(self):
        """
            Deletes all the maks from the map: bus stops,Hospitals,....
        """

        NodeL=self.getNodeList()
        childs=[]
        for element in NodeL:
            childs=element.childNodes
            nChilds=len(childs)
            if nChilds!=0:
                while nChilds!=0:
                    nChilds -=1
                    element.removeChild(childs[nChilds])
    
    def CreateOSMFile(self,Path,name='newmap.osm'):
        """
            It converts the internal structure into a OSM file and stores
            it in the Path and name given, if the file exists it will overwrite it.
        """

        Path=Path + '\\'+ name
        f=codecs.open(Path,'w',encoding='utf-8')
        self.root.writexml(f)
        f.close()

    def markNodes(self,List): # It has to be modified for general uses!!!!!!!!!!!!!!!!!!!!
        """
            Given a list of Nodes it adds a given mark to this Nodes.
        """
        for element in List:
            Node=self.getNode(element)
            mark=self.root.createElement('tag')
            mark.setAttribute('k','amenity')
            mark.setAttribute('v','hospital')
            Node.appendChild(mark)
            
    def getCoordinates(self,nodeID):
        """
            Given a nodeID it returns the Coordinates from that node in format [Longitude,Latitude]
        """
        if nodeID != '':
            node=self.getNode(nodeID)
            List=(node.getAttribute('lon'),node.getAttribute('lat'))
        return List

    def getPointsCoordinates(self):
        """
            Retursn a Dictionari with node ID as the index with Lon and Lat ans Values
        """
        nodes=self.getNodeList()
        coordinates={}
        for node in nodes:
            ID=node.getAttribute('id')
            coordinates[ID]=(node.getAttribute('lon'),node.getAttribute('lat'))
        return coordinates
            



#***********************************************************************************************************************************************************#
#********                                                          DATA HANDLER                                                                     ********#
#***********************************************************************************************************************************************************#
#                                                                                                                             #

class DataHandler():
    
    def CalculateStreetStadistics(self, Tree): #MODIFICAT!!!!!!!!!
        """
            It calculate the tortuosity,... of the streets in the Tree
        """

        streets=Tree.getStreetsList()
        Nodes=Tree.getIntersectionNodes()
        stadistics=osmStadistics(Nodes)
        graph=mapGraph(Tree.getStreetsList(),Tree.getNodeList())
        
        coordinates=Tree.getPointsCoordinates()
        for street in streets:
            elements=street.childNodes
            stadistics.calculateLevel(elements)
            stadistics.calculateTortuosity(elements,coordinates)
            stadistics.calculateDistanceMean(elements,coordinates)
            AMean=stadistics.calculateAngleMean(elements,coordinates)
            stadistics.calculateAngleVariance(elements,coordinates,AMean)
        
        stadistics.calculateIntersectionDistanceVariance(graph.getGraph(), coordinates)
        
        for intersec in Nodes:
            T=stadistics.getTortuosity(intersec)
            DM=stadistics.getDistanceMean(intersec)
            DV=stadistics.getDistanceVariance(intersec)
            AM=stadistics.getAngleMean(intersec)
            AV=stadistics.getAngleVariance(intersec)
            if len(T)>0 and len(DM)>0 and len(AM)>0 and len(AV)>0:
                stadistics.setValues(intersec,sum(T)/len(T),sum(DM)/len(DM),DV,sum(AM)/len(AM),sum(AV)/len(AV))
            else:
                stadistics.setValues(intersec,0,0,0,0,0)
        return stadistics,coordinates
    CalculateStreetStadistics=classmethod(CalculateStreetStadistics)

    def blocks(self, graph):
        """
            Given a graph it returns a list of loops
        """
        FinalblocksList=[]
        FinalblocksID=[]
        blocksList=[]
        blocksID=[]
        intersections=graph.intersectionPoints()

        for intersection in intersections:
            blocksID,blocksList=graph.circuits(intersection)
            for block in blocksList:
                FinalblocksList.append(block)
            for blockID in blocksID:
                FinalblocksID.append(blockID)
                
            blocksList=[]
        maxLen=0
        iterator=range(len(FinalblocksID))
        for i in iterator:
            if len(FinalblocksID[i])>len(FinalblocksID[maxLen]):
                maxLen=i
        del FinalblocksID[maxLen]
        del FinalblocksList[maxLen]
        return FinalblocksList,FinalblocksID
    
    blocks=classmethod(blocks)

    def streets(self, graph):
        keys=graph.Streets.keys()
        StreetsList=[]
        StreetsPList=[]
        for key in keys:
            PList=graph.Streets[key]
            StreetsList.append(PList)
            StreetsPList.append(graph.getCoordinatesList(PList))
        return StreetsPList,StreetsList
    streets=classmethod(streets)
    
#***********************************************************************************************************************************************************#
#********                                                   GENERIC GRAPH                                                                      ********#
#***********************************************************************************************************************************************************#
# undirected graph of all point                                                                                                                             #
from priodict import priorityDictionary

class mapGraph():
    def __init__(self, StreetsList, PointList):                             # Converts The Tree structure from osmTree into a graph structure
        self.Graph=priorityDictionary()#{}
        for point in PointList:
            self.Graph[point.getAttribute('id')]={}
        for street in StreetsList:
            StreetPoints=street.getElementsByTagName('nd')
            i=0
            streetLenght=len(StreetPoints)
            After=StreetPoints[1]
            Before=''
            while i<streetLenght:
                current=StreetPoints[i]
                edges=self.Graph[current.getAttribute('ref')]
                if After !='':
                    edges[After.getAttribute('ref')]=1                     #This is set to 1 because we want to search the minimum points circuit
                if Before !='':
                    edges[Before.getAttribute('ref')]=1
                Before=current
                i+=1
                if (i+1)>=streetLenght:
                    After=''
                else:
                    After=StreetPoints[i+1]

    def copy(self, originalGraph):                                                                  
        auxGraph=originalGraph.getGraph()
        self.Graph=auxGraph.copy()
                    
    def getGraph(self):                                                  # Returns the Graph
        return self.Graph

    def deleteEdge(self,Point1, Point2, Single=True):                                 # Given two points it deletes the edge betwen them if this exists
        if Point2 in self.Graph[Point1] :
            del self.Graph[Point1][Point2]
        if Point1 in self.Graph[Point2] and not Single:
            del self.Graph[Point2][Point1]
            
    def addEdgeList(self,Plist):
        FPoint=''
        for point in Plist:
            if FPoint=='':
                FPoint=point
            else:
                self.addEdge(FPoint,point)

    def addEdge(self,Point1, Point2):                                    # Given two points it creates a edge betwen them with cost1
        self.Graph[Point1][Point2]=1
        self.Graph[Point2][Point1]=1

    def setCost(self,Path,Cost):
        counter=1
        while counter<len(Path):
            self.Graph[Path[counter-1]][Path[counter]]=Cost
            self.Graph[Path[counter]][Path[counter-1]]=Cost
            counter+=1
        
    def deletePoint(self,Point):                                        # Given a point it deletes it and his edges
        pointList=self.Graph[Point]
        del self.Graph[Point]
        for element in pointList:
                del self.Graph[element][Point]
                
    def intersectionPoints(self):
        List=[]
        for point in self.Graph.keys():
            if len(self.Graph[point])>2:
                List.append(point)
        return List
    
    def Dijkstra(self,start,end=None):                                 # Dijkstra algorithm for minimum path betwen two points
        FinalDistances={}
        Predecessors={}
        noFinalVertexs=priorityDictionary()
        noFinalVertexs[start]=0

        for vertex in noFinalVertexs:
            FinalDistances[vertex]=noFinalVertexs[vertex]
            if vertex==end:break
            for element in self.Graph[vertex]:
                distance =FinalDistances[vertex] + self.Graph[vertex][element]
                if element in FinalDistances:
                    if distance < FinalDistances[element]:
                        raise ValueError, \
                              "found better path"
                elif (element not in noFinalVertexs or distance < noFinalVertexs[element]) and (vertex in self.Graph[element]) and (element in self.Graph[vertex]):
                    noFinalVertexs[element]=distance
                    Predecessors[element]=vertex
        return (FinalDistances,Predecessors)

    def shortestPath(self, start,end=None):                            # Given two points it return the shortest path betwen them, in format [startPoint,point1,point2,point3,...,endPoint]
        FinalDistances,Predecessors=self.Dijkstra(start,end)
        Path=[]
        while 1:
            Path.append(end)
            if end==start:break
            if not end in Predecessors:break
            end=Predecessors[end]
        Path.reverse()
        return Path

    def setEndgesStreet(self,StreetPoints):
        i=0
        streetLenght=len(StreetPoints)
        After=StreetPoints[1]
        Before=''
        while i<streetLenght:
            current=StreetPoints[i]
            edges=self.Graph[current]
            if After !='':
                  edges[After]=1                     #This is set to 1 because we want to search the minimum points circuit
            if Before !='':
                edges[Before]=1
            Before=current
            i+=1
            if (i+1)>=streetLenght:
                After=''
            else:
                After=StreetPoints[i+1]

    


#***********************************************************************************************************************************************************#
#********                                                   OSM MAP GRAPH                                                                           ********#
#***********************************************************************************************************************************************************#
# undirected graph of all point            
class osmGraph(mapGraph):
    def __init__(self, osmT):

        if osmT!='':
            mapGraph.__init__(self, osmT.getStreetsList(),osmT.getNodeList())
            self.Coordinates={}
            self.Streets={}
            for street in osmT.getStreetsList():
                streetPID=[]
                StreetPoints=street.getElementsByTagName('nd')
                for element in StreetPoints:
                    streetPID.append(element.getAttribute('ref'))
                self.Streets[street.getAttribute('id')]=streetPID
                
            for point in osmT.getNodeList():
                nodeID=point.getAttribute('id')
                self.Coordinates[nodeID]=osmT.getCoordinates(nodeID)
        else:
            mapGraph.__init__(self, [],[])
            self.Coordinates={}
            self.Streets={}

    def addNewPoint(self,Coordinates):
        keys=self.Coordinates.keys()
        keys.sort(reverse=True)
        Pid=str(int(keys[0])+1)
        self.Graph[Pid]={}
        self.Coordinates[Pid]=Coordinates
        return Pid
            
    def addPoint(self, pointID, Coordinates):
        if not pointID in self.Graph:
            self.Graph[pointID]={}
            self.Coordinates[pointID]=Coordinates

    def addPoints(self,IDList,CoordinatesDIC):
        lenght=len(IDList)
        i=0
        while i<lenght:
            self.addPoint(IDList[i],CoordinatesDIC[IDList[i]])
            i+=1
    def addStreet(self,PList):
        if self.Streets=={}:
            ID="1000000001"
        else:
            aux=self.Streets.keys()
            aux.sort(reverse=True)
            ID=str(int(aux[0])+1)
        self.Streets[ID]=PList
        
    def getCoordinates(self,nodeID):
        return self.Coordinates[nodeID]

    def getCoordinatesList(self,ListID):
        CList=[]
        for element in ListID:
            CList.append(self.getCoordinates(element))
        return CList
    
    def getStreets(self):
        return self.Streets

    def shortestPath(self, start,end=None):
        """
            Given two points it return the shortest path betwen them, in format [[CoordinateLonPointSTART,CoordinateLatPointSTART],[CoordinateLonPoint1,CoordinateLatPoint1],...,[CoordinateLonPointEND,CoordinateLatPointEND]]
        """
        FinalDistances,Predecessors=self.Dijkstra(start,end)
        Path=[]
        Coordinates=[]
        distance=-1
        if end in Predecessors:
            distance=FinalDistances[end]
        while 1:
            Coordinates.append(self.getCoordinates(end))
            Path.append(end)
            if end==start:break
            if not end in Predecessors:
                distance=-1
                break
            end=Predecessors[end]
        Path.reverse()
        Coordinates.reverse()
        return distance,Path,Coordinates
    
    def copy(self, originalGraph):
        """
            Given a graph it copy this into the current graph
        """
        auxGraph=originalGraph.getGraph()
        self.Graph=auxGraph.copy()
        self.Coordinates=originalGraph.Coordinates.copy()

    def circuits(self,Intersection):
        """
            Given a Intersection it returs a List of all ID and another with the coordinates of all the closet circuits that pass throught this point        
        """
        aTable=[]
        blocksID=[]
        blocksList=[]
        Sedges=[]
        Sedges.extend(self.Graph[Intersection])
        for edge in Sedges:
            Path=[Intersection]
            Mpoint=edge
            Spoint=Intersection
            self.deleteEdge(Spoint,Mpoint)
            points=[]
            points.extend(self.Graph[Mpoint])
            while(Mpoint!=Intersection):
                if points==[]:
                    Path=[]
                    break
                for point in points:
                    angle=geo.angle(self.Coordinates[Mpoint],self.Coordinates[Spoint],self.Coordinates[point])
                    if angle>math.pi:
                        angle=angle-2*math.pi
                    aTable.append([angle,point])
                Path.append(Mpoint)
                Spoint=Mpoint
                Mpoint=geo.minLeftangle(aTable)[1]
                self.deleteEdge(Spoint,Mpoint)
                aTable=[]
                points=[]
                points.extend(self.Graph[Mpoint])

            if Path!=[]:
                blocksID.append(Path)
                blocksList.append(self.getCoordinatesList(Path))
                
        return blocksID, blocksList

    def selectNodesArea(self,point1,point2):
        """
            given two points it returns a dictionari with the ID as the key and the coordinates as the value with the points contained within the coordinates of the two given.
            point1 and point2 in format [longitude,latitude]
        """
        result={}
        lonP1=float(point1[0])
        lonP2=float(point2[0])
        latP1=float(point1[1])
        latP2=float(point2[1])
        points=self.Coordinates.keys()
        for point in points:
            coord=(float(self.Coordinates[point][0]),float(self.Coordinates[point][1]))
            if ((lonP1<=coord[0] and coord[0]<=lonP2) or (lonP1>=coord[0] and coord[0]>=lonP2)) and ((latP1<=coord[1] and coord[1]<=latP2) or (latP1>=coord[1] and coord[1]>=latP2)) :
                result[point]=self.Coordinates[point]
        return result

    def getSubGraph(self,point1,point2):
        CoordinatesNodes=self.selectNodesArea(point1,point2)
        Nodes=CoordinatesNodes.keys()
        subGraph=osmGraph('')
        subGraph.addPoints(Nodes,CoordinatesNodes)
        OriginalStreets=self.Streets.keys()
        for OStreet in OriginalStreets:
            OPoints=self.Streets[OStreet]
            NPoints=[]
            for point in OPoints:
                if point in Nodes:
                    NPoints.append(point)
            if len(NPoints)>1:
                subGraph.Streets[OStreet]=NPoints
                subGraph.setEndgesStreet(NPoints)
                
        return subGraph

    def GenerateOSMTree(self,StreetsList=[]):  #modificat
        """
            
        """
        cityOSM=osmTree('')
        cityOSM.addNodes(self.Coordinates)
        keys=self.Streets.keys()
        for street in keys:
            cityOSM.addStreet(["%s" % el for el in self.Streets[street]],str(street)) #http://www.daniweb.com/forums/thread93655.html#

        cityOSM.setBounds()
        return cityOSM
        
        


#***********************************************************************************************************************************************************#
#********                                                   Dictionari Stadistics                                                                   ********#
#***********************************************************************************************************************************************************#
# Stadistics for the osm Files                                                                                                                              #

class osmStadistics():
    __streetHierarchy={  "motorway":9,            # Street hierachy for just to determinate intersection level.
                         "motorway_link":9,
                         "trunk":8,
                         "trunk_link":8,
                         "primary":7,
                         "primary_link":7,
                         "secondary":6,
                         "secondary_link":6,
                         "tertiary":5,
                         "unclassified":4,
                         "road":3,
                         "residential":2,
                         "living_street":1,
                         "service":1,
                        }
        
    def __init__(self,List):
        self.Intersections={}
        self.Level={}                                                                   # L'index ser el Nivell i contidr les ID dels Carrers amb aquell nivell.
        for element in List:
            self.Intersections[element]=[[],[],[],[],[],[]]                             #[0]=tortuosity, [1]=dmean, [2]=dVariance, [3]=anglemean, [4]=angleVariance

    def setValues(self,ID,tortuosity,dmean,dVariance,anglemean,angleVariance):
        self.Intersections[ID]=[tortuosity,dmean,dVariance,anglemean,angleVariance]

    def deletePoint(self,ID):
        if ID in self.Intersections:
            del self.Intersections[ID]
        if ID in self.Level:
            del self.Level[ID]

    def setLevelList(self, Iid, levelList):
        self.Intersections[Iid].setLevelList(levelList)

    def calculateLevel(self,streetPoints):
        intersections=[]
        level=0
        for point in streetPoints:
            if point.nodeType!=3:
                nodeID= point.getAttribute('ref')
                if nodeID in self.Intersections:
                    intersections.append(nodeID)
                elif nodeID=='':
                    k= point.getAttribute('k')
                    if k=='highway':
                        levelStr= point.getAttribute('v')
                        if levelStr in osmStadistics.__streetHierarchy:
                            level=osmStadistics.__streetHierarchy[levelStr]
        for element in intersections:
            self.addLevel(element,level)
                
        
    def setTortuosityList(self, Iid, tortuosityList):
        self.Intersections[Iid].setTortuosityList(tortuosityList)

    def calculateTortuosity(self,streetPoints,coordinates):
        lastIntersection=''
        firstIntersection=''
        lastNode=''
        intersectionDistance=0
        intersections=[]
        tortuosity=0
        nSegments=0
        for point in streetPoints:
            if point.nodeType!=3:

                nodeID= point.getAttribute('ref')
                if nodeID in coordinates:
                    if lastIntersection=='': 
                       firstIntersection= lastNode= lastIntersection= nodeID
                       #intersections.append(nodeID)
               
                    elif nodeID!=lastIntersection:
                        nodeCoordinates=coordinates[nodeID]
                        lastCoordinates=coordinates[lastNode]
                        distance=geo.distance(float(lastCoordinates[0]),float(lastCoordinates[1]),float(nodeCoordinates[0]),float(nodeCoordinates[1]))
                        lastNode=nodeID
                        intersectionDistance=intersectionDistance+distance
                        if nodeID in self.Intersections:
                            intersections.append(nodeID)
                            nodeCoordinates=coordinates[nodeID]
                            lastCoordinates=coordinates[lastIntersection]
                            tortuosity=tortuosity + intersectionDistance/(geo.distance(float(lastCoordinates[0]),float(lastCoordinates[1]),float(nodeCoordinates[0]),float(nodeCoordinates[1])))
                            intersectionDistance=0
                            lastIntersection=nodeID
                            nSegments+=1
        intersections.sort()
        if nSegments!=0:
            tortuosity=tortuosity/nSegments
        else:
            tortuosity=0
        for element in intersections:
            self.addTortuosity(element,tortuosity)

    def setDistanceMean(self, Iid, distanceMean):
        self.Intersections[Iid].setDistanceMean(distanceMean)

    def calculateDistanceMean(self,streetPoints,coordinates):
        """
        """
        streetIntersections=[]
        distance=0
        SegmentsLenght=0
        previousNode=''
        Mean=0
        FIntersection=''
        nodeCoordinates=[]
        previousCoordinates=[]
        for point in streetPoints:
            if point.nodeType!=3:
                nodeID= point.getAttribute('ref')
                if nodeID in self.Intersections and previousCoordinates==[]:
                    FIntersection=nodeID
                    previousCoordinates=coordinates[nodeID]
                    
                elif nodeID in self.Intersections and nodeCoordinates!=[]:
                    streetIntersections.append(nodeID)
                    nodeCoordinates=coordinates[nodeID]
                    distance+=geo.distance(float(previousCoordinates[0]),float(previousCoordinates[1]),float(nodeCoordinates[0]),float(nodeCoordinates[1]))
                    previousCoordinates=coordinates[nodeID]
                    SegmentsLenght+=distance
                    distance=0
                    
                elif previousCoordinates!=[] and nodeID!='':
                    nodeCoordinates=coordinates[nodeID]
                    distance+=geo.distance(float(previousCoordinates[0]),float(previousCoordinates[1]),float(nodeCoordinates[0]),float(nodeCoordinates[1]))
                    previousCoordinates=coordinates[nodeID]
                    
        if len(streetIntersections)!=0:
            Mean=SegmentsLenght/len(streetIntersections)
            streetIntersections.append(FIntersection)

        for element in streetIntersections:
            self.addDistanceMean(element,Mean)
        return Mean
        
    def setDistanceVariance(self, Iid, distanceVariance):
        sself.Intersections[Iid].setDistanceVariance(distanceVariance)

    def calculateDistanceVariance(self,streetPoints,coordinates,Mean):#MODIFICAR!!!!!!!!
        """
        """
        streetIntersections=[]
        variance=0
        distance=0
        previousNode=''
        FIntersection=''
        nodeCoordinates=[]
        previousCoordinates=[]
        for point in streetPoints:
            if point.nodeType!=3:
                nodeID= point.getAttribute('ref')
                if nodeID in self.Intersections and previousCoordinates==[]:
                    FIntersection=nodeID
                    previousCoordinates=coordinates[nodeID]
                    
                elif nodeID in self.Intersections and nodeCoordinates!=[]:
                    streetIntersections.append(nodeID)
                    nodeCoordinates=coordinates[nodeID]
                    distance+=geo.distance(float(previousCoordinates[0]),float(previousCoordinates[1]),float(nodeCoordinates[0]),float(nodeCoordinates[1]))
                    previousCoordinates=coordinates[nodeID]
                    variance+=(distance-Mean)**2
                    distance=0
                    
                elif previousCoordinates!=[] and nodeID!='':
                    nodeCoordinates=coordinates[nodeID]
                    distance+=geo.distance(float(previousCoordinates[0]),float(previousCoordinates[1]),float(nodeCoordinates[0]),float(nodeCoordinates[1]))
                    previousCoordinates=coordinates[nodeID]
                    
        if len(streetIntersections)!=0:
            variance=variance/len(streetIntersections)
            streetIntersections.append(FIntersection)
        for element in streetIntersections:
            self.addDistanceVariance(element,variance)

    def calculateIntersectionDistanceVariance(self,graph,coordinates): #graph=diccionari
        keys=self.Intersections.keys()
        for key in keys:
            DM=self.getDistanceMean(key)
            if DM ==[]:
                self.Intersections[key][3]=0
            else:
                DM=sum(DM)/len(DM)
                Fpoints=graph[key]
                CpointCoordinates=coordinates[key]
                variance=0
                for point in Fpoints:
                    pointCoordinates=coordinates[point]
                    distance=geo.distance(float(pointCoordinates[0]),float(pointCoordinates[1]),float(CpointCoordinates[0]),float(CpointCoordinates[1]))
                    variance+=(distance-DM)**2
                    
                self.Intersections[key][3]=variance/len(Fpoints)
        
    def setAngleMean(self, Iid, angleMean):
        self.Intersections[Iid].setAngleMean(angleMean)

    def calculateAngleMean(self,streetPoints,coordinates):
        """
        """
        streetIntersections=[]
        angle=0
        Mean=0
        SNode=[]
        MNode=[]
        nAngles=0
        FIntersection=''
        for point in streetPoints:
            if point.nodeType!=3:
                nodeID= point.getAttribute('ref')
                if nodeID in self.Intersections and SNode==[]:
                    streetIntersections.append(nodeID)
                    SNode=coordinates[nodeID]
                    
                elif nodeID in self.Intersections and MNode!=[]:
                    nAngles+=1
                    streetIntersections.append(nodeID)
                    FNode=coordinates[nodeID]
                    angle+=geo.angle(MNode,SNode,FNode)
                    SNode=MNode
                    MNode=FNode

                elif SNode!=[] and MNode==[] and nodeID!='':
                    MNode=coordinates[nodeID]
                
                elif SNode!=[] and nodeID!='':
                    nAngles+=1
                    FNode=coordinates[nodeID]
                    angle+=geo.angle(MNode,SNode,FNode)
                    SNode=MNode
                    MNode=FNode

        if nAngles!=0:
            Mean=angle/nAngles
            if Mean<0:
                Mean=2*math.pi+Mean

        for element in streetIntersections:
            self.addAngleMean(element,Mean)
        return Mean
        
    def setAngleVariance(self, Iid, angleVariance):
        self.Intersections[Iid].setAngleVariance(angleVariance)
        
    def calculateAngleVariance(self,streetPoints,coordinates,Mean):
        """
        """
        streetIntersections=[]
        variance=0
        angle=0
        SNode=[]
        MNode=[]
        nAngles=0
        for point in streetPoints:
            if point.nodeType!=3:
                nodeID= point.getAttribute('ref')
                if nodeID in self.Intersections and SNode==[]:
                    streetIntersections.append(nodeID)
                    SNode=coordinates[nodeID]
                    
                elif nodeID in self.Intersections and MNode!=[]:
                    nAngles+=1
                    streetIntersections.append(nodeID)
                    FNode=coordinates[nodeID]
                    angle=geo.angle(MNode,SNode,FNode)
                    if angle<0:
                        angle=2*math.pi+angle
                    variance+=(angle-Mean)**2
                    SNode=MNode
                    MNode=FNode

                elif SNode!=[] and MNode==[] and nodeID!='':
                    MNode=coordinates[nodeID]
                
                elif SNode!=[] and nodeID!='':
                    nAngles+=1
                    FNode=coordinates[nodeID]
                    angle=geo.angle(MNode,SNode,FNode)
                    if angle<0:
                        angle=2*math.pi+angle
                    variance+=(angle-Mean)**2
                    SNode=MNode
                    MNode=FNode

        if nAngles!=0:
            variance=variance/nAngles
        
        for element in streetIntersections:
            self.addAngleVariance(element,variance)
        
    def addLevel(self, Iid, level):                                 # El primer pas el deixo per s'ha de mirar si es po eliminar!!!!!!!!!!!
        L=self.getLevelList(Iid)
        L.append(level)
        self.Intersections[Iid][0]=L
        if level in self.Level:
            copy=self.Level[level]
            copy.append(Iid)
            self.Level[level]=copy
        else:
            self.Level[level]=[Iid]
    
    def addTortuosity(self, Iid, tortuosity):
        L=self.getTortuosity(Iid)
        L.append(tortuosity)
        self.Intersections[Iid][1]=L

    def addDistanceMean(self, Iid, Mean):
        L=self.getDistanceMean(Iid)
        L.append(Mean)
        self.Intersections[Iid][2]=L

    def addDistanceVariance(self, Iid, Variance):
        L=self.getDistanceVariance(Iid)
        L.append(Variance)
        self.Intersections[Iid][3]=L
    
    def addAngleMean(self, Iid, Mean):
        L=self.getAngleMean(Iid)
        L.append(Mean)
        self.Intersections[Iid][4]=L

    def addAngleVariance(self, Iid, Variance):
        L=self.getAngleVariance(Iid)
        L.append(Variance)
        self.Intersections[Iid][5]=L

    def getLevelList(self, Iid):
        return self.Intersections[Iid][0]

    def getTortuosity(self, Iid):               #s'ha eliminat List del nom d'aquest i els segents.
        return self.Intersections[Iid][1]

    def getDistanceMean(self, Iid):
        return self.Intersections[Iid][2]

    def getDistanceVariance(self, Iid):
        return self.Intersections[Iid][3]
    
    def getAngleMean(self, Iid):
        return self.Intersections[Iid][4]

    def getAngleVariance(self, Iid):
        return self.Intersections[Iid][5]
    
#***********************************************************************************************************************************************************#
#********                                                   Geographic Calculations                                                                 ********#
#***********************************************************************************************************************************************************#
# Group of various methods for geographic calculation                                                                                                       #

class geo():
    def distance(self,pointALon,pointALat,pointBLon,pointBLat): 
        Radius = 6371 # km
        Lat = math.radians(pointBLat-pointALat)
        Lon = math.radians(pointBLon-pointALon) 
        a = math.sin(Lat/2) * math.sin(Lat/2) + math.cos(math.radians(pointALat)) * math.cos(math.radians(pointBLat)) * math.sin(Lon/2) * math.sin(Lon/2) 
        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a)) 
        dist = Radius * c
        return dist
    distance=classmethod(distance)

    def aDistance(self,pointALon,pointALat,pointBLon,pointBLat):
        D=-1
        Lat=pointBLat-pointALat
        Lon=pointBLon-pointALon
        if Lat+Lon==0:
            print "0 VECTOR!!!!!!!!!!"
        else:
            D=math.sqrt((Lat**2)+(Lon**2))
        return D
    aDistance=classmethod(aDistance)

    def angle(self,MCoordinates,SCoordinates,FCoordinates):#http://www.euclideanspace.com/maths/algebra/vectors/angleBetween/index.htm
        if SCoordinates !='':
            Vector1=[((float(MCoordinates[0])-float(SCoordinates[0]))),((float(MCoordinates[1])-float(SCoordinates[1])))]
            Vector2=[((float(MCoordinates[0])-float(FCoordinates[0]))),((float(MCoordinates[1])-float(FCoordinates[1])))]
            angle=math.atan2(Vector2[1],Vector2[0]) - math.atan2(Vector1[1],Vector1[0])
        else:
            angle=0
        return angle
    angle=classmethod(angle)

    def SegmentIntersection(self,A,B,C,D):          #http://local.wasp.uwa.edu.au/~pbourke/geometry/lineline2d/
        Intersect=False
        Denominator=(D[1]-C[1])*(B[0]-A[0])-(D[0]-C[0])*(B[1]-A[1])
        if Denominator!=0:
            Ua=((D[0]-C[0])*(A[1]-C[1])-(D[1]-C[1])*(A[0]-C[0]))/Denominator
            Ub=((B[0]-A[0])*(A[1]-C[1])-(B[1]-A[1])*(A[0]-C[0]))/Denominator
            if (0<=Ua and Ua<=1) and (0<=Ub and Ub<=1):
                Intersect=True
        return Intersect
    SegmentIntersection=classmethod(SegmentIntersection)

    
    def DPointSegment(self,A,B,C):
        Result=0
        if A[0]==B[0] and A[1]==B[1]:
            Result = geo.distance(C[0], C[1], A[0], A[1]);
        else:
            Vx= B[0] - A[0];
            Vy= B[1] - A[1];
            Ratio = ((C[0] - A[0]) * Vx + (C[1] - A[1]) * Vy) / (Vx**2 + Vy**2);
            if Ratio < 0 :
                Result = geo.distance(C[0], C[1], A[0], A[1])
            elif Ratio > 1 :
                Result = geo.distance(C[0], C[1], B[0], B[1])
            else:
                Result = geo.pdis(A,B,C)#geo.distance(C[0], C[1], (1 - Ratio) * A[0] + Ratio * B[0],(1 - Ratio) * A[1] + Ratio * B[1]);
        return Result
    DPointSegment=classmethod(DPointSegment)

    def overlap(self,A,B,C,D):
        a=0
    overlap=classmethod(overlap)

#    from math import sqrt, fabs
    def pdis(self,a, b, c):                     #http://local.wasp.uwa.edu.au/~pbourke/geometry/pointline/
        u=((c[0]-a[0])*(b[0]-a[0])+(c[1]-a[1])*(b[1]-a[1]))/((b[0]-a[0])**2 + (b[1]-a[1])**2)
        x = a[0] + u*(b[0]-a[0])
        y = a[1] + u*(b[1]-a[1])
        return geo.distance(c[0],c[1],x,y)
        
    pdis=classmethod(pdis)

    def minLeftangle(self,Table):
        """
            Table in format [[angle0,...],[angle1,...],...]
        """
        minimum=Table[0]
        if minimum[0]<=0:
            minimum[0]=2*math.pi+minimum[0]
        
        for element in Table:
            if element[0]<=0:
                element[0]=2*math.pi+element[0]

            if element[0]<minimum[0]:
                minimum=element
                
        return minimum
    minLeftangle=classmethod(minLeftangle)

    def PointAngularDistanceStartLine(self,PLStart,PLEnd,ADistance):
        Point=None
        XR=PLEnd[0]-PLStart[0]
        YR=PLEnd[1]-PLStart[1]
        if XR+YR==0:
            print "0 VECTOR!!!!!!!!!!"
        else:
            u=abs(math.sqrt((ADistance**2)/((XR**2)+(YR**2))))
            Point=((PLStart[0]+u*XR),(PLStart[1]+u*YR))
        return Point
    PointAngularDistanceStartLine=classmethod(PointAngularDistanceStartLine)

    def perpendicularVector(self,Vector):
        return (-1*Vector[1],Vector[0])
    perpendicularVector=classmethod(perpendicularVector)

#***********************************************************************************************************************************************************#
#********                                                   Bivariate Gaussian                                                                       ********#
#***********************************************************************************************************************************************************#
#                                                                                                                                                           #

class BiGauss():
    def provability(self,x,y,meanX,meanY,VarianceX,VarianceY):
        prov=0
        if VarianceX!=0 and VarianceY!=0:  #per evitar divisio per 0
            prov= math.exp(-(((x-meanX)**2)/(2*(VarianceX))+((y-meanY)**2)/(2*(VarianceY))))#(1/(2*math.pi*VarianceX*VarianceY))*math.exp(-(((x-meanX)**2)/(2*(VarianceX**2))+((y-meanY)**2)/(2*(VarianceY**2)))) 
        return prov
    provability=classmethod(provability)

#***********************************************************************************************************************************************************#
#********                                                              CDF                                                                          ********#
#***********************************************************************************************************************************************************#
#                                                                                                                                                           #    

class CDF():
    def cdf(self,PList):
        """
            PList in format [[Provability,ID],...]
        """
        Point=''
        if PList!=[]:
            PList.sort(reverse=True)
            aux=0
            posList=range(len(PList))
            for pos in posList:
                aux=aux+PList[pos][0]
                PList[pos][0]=aux
            rand=random.random()*PList[(len(PList)-1)][0]
            Point=PList[0][1]
            for pos2 in posList:
                if rand>PList[pos2][0]:
                    Point=PList[pos2][1]
        return Point
    cdf=classmethod(cdf)

#***********************************************************************************************************************************************************#
#********                                                          Street Generator                                                                 ********#
#***********************************************************************************************************************************************************#
#                                                                                                        #
import random

class streetGenerator():
    def __init__(self,List,Coordinates,PointsLevel):
        """
           
        """
        self.nUsedIntersection={}
        self.LevelIntersection=PointsLevel.copy()
        self.Coordinates=Coordinates.copy()
        self.Stadistics=List.copy()
        self.Edges={}
        keys=List.keys()
        for key in keys:
            self.nUsedIntersection[key]=0
            
    def addNewPointCoordinates(self,Coordinates):# NOU
        keys=self.Coordinates.keys()
        keys.sort(reverse=True)
        Pid=str(int(keys[0])+1)
        self.Coordinates[Pid]=Coordinates
        return Pid
    
    def addEdge(self,Point1,Point2):
        if Point1 in self.Edges:
            aux=self.Edges[Point1]
            aux.append(Point2)
            self.Edges[Point1]=aux
        else:
            self.Edges[Point1]=[Point2]
            
    def deletePointCoordinates(self,PID):# NOU
        if PID in self.Coordinates:
            del self.Coordinates[PID]
    
    def crossEdge(self,Cpoint,Npoint):
        cross=False
        Edges=self.Edges.keys()
        for edge in Edges:
            if edge!=Npoint and edge!=Cpoint:
                a=[float(self.Coordinates[edge][0]),float(self.Coordinates[edge][1])]
                Fpoints=self.Edges[edge]
                for Fpoint in Fpoints:
                    if Fpoint!=Npoint and Fpoint!=Cpoint:
                        b=[float(self.Coordinates[Fpoint][0]),float(self.Coordinates[Fpoint][1])]
                        c=[float(self.Coordinates[Cpoint][0]),float(self.Coordinates[Cpoint][1])]
                        d=[float(self.Coordinates[Npoint][0]),float(self.Coordinates[Npoint][1])]
                        if geo.SegmentIntersection(a,b,c,d):# or ([Cpoint,Npoint] in Edges) or ([Npoint,Cpoint] in Edges): #Si interseca o ja existeix
                            x=[edge,Fpoint]
                            cross=True
                            break
        return cross

    def tooSimilar(self,Cpoint,Npoint):
        Cp=False
        Np=False
        if Cpoint in self.Edges:
            Tedges=self.Edges[Cpoint]
            for edge in Tedges:
                angle=geo.angle(self.Coordinates[Cpoint],self.Coordinates[Npoint],self.Coordinates[edge])
                if Configuration.MinStreetsAngle>angle and angle>-Configuration.MinStreetsAngle:
                    Cp=True
                    break
        if Npoint in self.Edges:
            Tedges=self.Edges[Npoint]
            for edge in Tedges:
                angle=geo.angle(self.Coordinates[Npoint],self.Coordinates[Cpoint],self.Coordinates[edge])
                if Configuration.MinStreetsAngle>angle and angle>-Configuration.MinStreetsAngle:                                                                                #LiMITO L'angle entr dos carrers a 0.4 rad
                    Np=True
                    break
                
        return (Cp or Np)

    def crossPoint(self,Cpoint,Npoint):
        Cross=False
        Points=self.nUsedIntersection.keys()
        a=[float(self.Coordinates[Cpoint][0]),float(self.Coordinates[Cpoint][1])]
        b=[float(self.Coordinates[Npoint][0]),float(self.Coordinates[Npoint][1])]
        for point in Points:
            if point != Cpoint and point !=Npoint:
                c=[float(self.Coordinates[point][0]),float(self.Coordinates[point][1])]
                distance=geo.DPointSegment(a,b,c)
                if distance<Configuration.MinStreetsDistance:
                    Cross=True
                    break
        return Cross

    def choosePoint(self,Cpoint,PointsList,Ppoint,street):
        SelectablePoints=[]
        sPoints=[]
        Npoint=''
        for Point in PointsList:
            creua=self.crossEdge(Cpoint,Point)
            distance=geo.distance(float(self.Coordinates[Cpoint][0]),float(self.Coordinates[Cpoint][1]),float(self.Coordinates[Point][0]),float(self.Coordinates[Point][1]))
            if Ppoint=='':
                angle=self.Stadistics[Cpoint][3]#Cpoint
            else:
                angle=geo.angle(self.Coordinates[Cpoint],self.Coordinates[Ppoint],self.Coordinates[Point])
            if angle<0:
                angle=2*math.pi+angle
            goodangle= angle>Configuration.MaxSegmentAngle and angle<((math.pi*2)-Configuration.MaxSegmentAngle)
            if Configuration.MaxSegmentLenght==0 or (Configuration.MaxSegmentLenght!=0 and distance<Configuration.MaxSegmentLenght) :
                if Point!=Cpoint and self.nUsedIntersection[Point]<Configuration.MaxSegmentsIntersection and not creua and not self.tooSimilar(Cpoint,Point) and not self.crossPoint(Cpoint,Point) and goodangle:
                    if Ppoint!='':
                        angle=geo.angle(self.Coordinates[Cpoint],self.Coordinates[Ppoint],self.Coordinates[Point])
                    else:
                        angle=self.Stadistics[Cpoint][3]
                    Prob=BiGauss.provability(distance,angle,self.Stadistics[Cpoint][1],self.Stadistics[Cpoint][3],self.Stadistics[Cpoint][2],self.Stadistics[Cpoint][4])
                    if Prob>Configuration.MinSegmentProvability and not (Point in sPoints)and not(Point in street):
                        SelectablePoints.append([Prob,Point])
                        sPoints.append(Point)
        Npoint=CDF.cdf(SelectablePoints)
        return Npoint


    def avariableSlots(self,PointsList):
        avariable=False
        for Point in PointsList:
            if self.nUsedIntersection[Point]<Configuration.MaxSegmentsIntersection:
                avariable=True
                break
        return avariable

    def generateStreets(self):
        city=osmGraph('')
        city.addPoints(self.Coordinates.keys(),self.Coordinates)
        levels=self.LevelIntersection.keys()
        levels.sort(reverse=True)
        Nstreets=[]
        for level in levels:
            LStreets=[]
            Points=self.LevelIntersection[level]
            Continue=True
            while self.avariableSlots(self.LevelIntersection[level]) and Continue:
                Continue=False
                for point in Points:
                    Auxstreets=self.randomWalk(point,self.LevelIntersection[level],city)
                    if len(Auxstreets)>1:
                        Continue=True
                        Nstreets.append(Auxstreets)
                        city.addStreet(Auxstreets)
                        LStreets.append(Auxstreets)
        return city,Nstreets

    def randomWalk(self,Point,PointsList,city):
        stop=False
        street=[]
        Fpoint=False
        Flenght=0
        positive=True
        if len(PointsList)>1:
            Cpoint=Point
            Ppoint=''
            street=[Cpoint]
            while not stop and self.nUsedIntersection[Cpoint]<4:
                Npoint=self.choosePoint(Cpoint,PointsList,Ppoint,street)
                if Npoint=='':
                    stop=True
                else:
                    Plist=self.generateEdgeTortuosity([Cpoint, Npoint],positive,Configuration.TortuosityPoints)
                    positive=not positive
                    start=Cpoint
                    for Point in Plist:
                        p=city.addNewPoint((str(Point[0]),str(Point[1])))
                        city.addEdge(start, p)
                        street.append(p)
                        start=p
                    street.append(Npoint)
                    city.addEdge(start, Npoint)
                    self.nUsedIntersection[Cpoint]+=1
                    self.nUsedIntersection[Npoint]+=1
                    Ppoint=Cpoint
                    Cpoint=Npoint
        return street

    def generateEdgeTortuosity(self, Edge,positive, nPoints=2, SecondTry=False):
        Lm=geo.distance(float(self.Coordinates[Edge[0]][0]),float(self.Coordinates[Edge[0]][1]),float(self.Coordinates[Edge[1]][0]),float(self.Coordinates[Edge[1]][1]))
        T=(self.Stadistics[Edge[0]][0]+self.Stadistics[Edge[1]][0])/2
        Dm=Lm*T
        L=geo.aDistance(float(self.Coordinates[Edge[0]][0]),float(self.Coordinates[Edge[0]][1]),float(self.Coordinates[Edge[1]][0]),float(self.Coordinates[Edge[1]][1]))
        D=L*T
        Plist=[]
       
        if nPoints==2:
            bsd=D/3
            ssd=(L-(bsd))/2
            if bsd!=0:
                H=math.sqrt(abs(ssd**2+bsd**2-2*ssd*bsd*(ssd/bsd)))
                point1=geo.PointAngularDistanceStartLine((float(self.Coordinates[Edge[0]][0]),float(self.Coordinates[Edge[0]][1])),(float(self.Coordinates[Edge[1]][0]),float(self.Coordinates[Edge[1]][1])),ssd)
                point2=geo.PointAngularDistanceStartLine((float(self.Coordinates[Edge[1]][0]),float(self.Coordinates[Edge[1]][1])),(float(self.Coordinates[Edge[0]][0]),float(self.Coordinates[Edge[0]][1])),ssd)
                if positive:
                    Vect1=(point1[0]-(float(self.Coordinates[Edge[0]][0])), point1[1]-(float(self.Coordinates[Edge[0]][1])))
                    Vect2=((float(self.Coordinates[Edge[1]][0])-point2[0]), (float(self.Coordinates[Edge[1]][1]))-point2[1])
                else:
                    Vect1=((float(self.Coordinates[Edge[0]][0])-point1[0]), (float(self.Coordinates[Edge[0]][1])-point1[1]))
                    Vect2=(point2[0]-(float(self.Coordinates[Edge[1]][0])), (point2[1]-float(self.Coordinates[Edge[1]][1])))
                    
                PVect1=geo.perpendicularVector(Vect1)
                PVect2=geo.perpendicularVector(Vect2)
                Paux1=(point1[0]+PVect1[0],point1[1]+PVect1[1])
                Paux2=(point2[0]+PVect2[0],point2[1]+PVect2[1])
                P1=geo.PointAngularDistanceStartLine(point1,Paux1,H)            # Problemes de poca exactitut
                P2=geo.PointAngularDistanceStartLine(point2,Paux2,H)            # Problemes de poca exactitut
                p1=self.addNewPointCoordinates(P1)
                p2=self.addNewPointCoordinates(P2)
                
                if self.crossEdge(Edge[0],p1) or self.crossPoint(Edge[0],p1) or self.crossEdge(p1,p2) or self.crossPoint(Edge[0],p1) or self.crossEdge(p2,Edge[1]) or self.crossPoint(Edge[0],p1):
                    if SecondTry:
                        self.addEdge(Edge[0],Edge[1])
                        self.addEdge(Edge[1],Edge[0])
                        PList=[]
                    else:
                        PList=self.generateEdgeTortuosity(Edge,positive, nPoints, True)
                else:
                    self.addEdge(Edge[0],p1)
                    self.addEdge(p1,Edge[0])
                    self.addEdge(p1,p2)
                    self.addEdge(p2,p1)
                    self.addEdge(p2,Edge[1])
                    self.addEdge(Edge[1],p2)
                    Plist=[P1,P2]
            
        elif nPoints==1:
            ssd=L/2
            bsd=D/2
            H=math.sqrt(abs(ssd**2+bsd**2-2*ssd*bsd*(ssd/bsd)))
            point1=geo.PointAngularDistanceStartLine((float(self.Coordinates[Edge[0]][0]),float(self.Coordinates[Edge[0]][1])),(float(self.Coordinates[Edge[1]][0]),float(self.Coordinates[Edge[1]][1])),ssd)
            if bsd!=0:
                
                if positive:
                    Vect1=(point1[0]-(float(self.Coordinates[Edge[0]][0])), point1[1]-(float(self.Coordinates[Edge[0]][1])))
                else:
                    Vect1=((float(self.Coordinates[Edge[0]][0])-point1[0]), (float(self.Coordinates[Edge[0]][1])-point1[1]))
                    
                PVect1=geo.perpendicularVector(Vect1)
                Paux1=(point1[0]+PVect1[0],point1[1]+PVect1[1])
                P1=geo.PointAngularDistanceStartLine(point1,Paux1,H)            # Problemes de poca exactitut
                p1=self.addNewPointCoordinates(P1)
               
                if self.crossEdge(Edge[0],p1) or self.crossPoint(Edge[0],p1) or self.crossEdge(p1,Edge[1]) or self.crossPoint(Edge[0],p1):
                    if SecondTry:
                        PList=[]
                    else:
                        PList=self.generateEdgeTortuosity(Edge,positive, nPoints, True)
                else:
                    Plist=[P1]
        return Plist

    def generateTortuosity(self,nPoints=2):
        Edges=self.Edges.keys()
        for edge in Edges:
            Points=self.Edges[edge]
            positive=True
            for point in Points:
                self.generateEdgeTortuosity([edge,point],positive,nPoints)
                positive=not positive

        
        



#***********************************************************************************************************************************************************#
#********                                                          Configuration                                                                 ********#
#***********************************************************************************************************************************************************#
#                                                                                                        #
class Configuration():
    MinStreetsAngle=0.3      #
    MinStreetsDistance=0.02  #
    MaxSegmentAngle=math.pi/2
    MaxSegmentsIntersection=4#
    MinSegmentProvability=0  #
    MaxSegmentLenght=0
    TortuosityPoints=2
    
            


#path = 'D:\PFC'
#path = 'F:\PFC'

#import time
#*******************************************************************************
#****                           MAIN                                        ****
#*******************************************************************************

#test=osmTree(path+"\mapes\BCN2.osm")
#test.eraseNoTypeElements(u'highway')
#test.deleteNodeMarks()
#a=osmGraph(test)
#test.markNodes(['426890148'])
#test.CreateOSMFile("c:")
#test.markNodes(a.Coordinates.keys())
#test.CreateOSMFile(path)
#print DataHandler.blocks(a)
